package com.flow.flowable.configure;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.flowable.common.engine.impl.cfg.IdGenerator;
import org.flowable.common.engine.impl.persistence.deploy.DeploymentCache;
import org.flowable.engine.impl.persistence.deploy.ProcessDefinitionCacheEntry;
import org.flowable.spring.SpringProcessEngineConfiguration;
import org.flowable.spring.boot.EngineConfigurationConfigurer;
import org.flowable.task.service.impl.DefaultTaskPostProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.PlatformTransactionManager;

import java.util.Objects;

@Configuration
public class FlowableConfigure implements EngineConfigurationConfigurer<SpringProcessEngineConfiguration> {
    @Autowired(required = false)
    private PlatformTransactionManager transactionManager;
    @Autowired(required = false)
    private ObjectMapper objectMapper;
    @Autowired(required = false)
    private IdGenerator idGenerator;
    @Autowired(required = false)
    private DefaultTaskPostProcessor taskPostProcessor;
    @Autowired(required = false)
    private DeploymentCache<ProcessDefinitionCacheEntry> deploymentCache;

    @Override
    public void configure(SpringProcessEngineConfiguration engineConfiguration) {
        if (Objects.nonNull(this.objectMapper)) {
            engineConfiguration.setObjectMapper(objectMapper);
        }
        if (Objects.nonNull(this.idGenerator)) {
            engineConfiguration.setIdGenerator(idGenerator);
        }
        if (Objects.nonNull(taskPostProcessor)) {
            engineConfiguration.setTaskPostProcessor(taskPostProcessor);
        }
        if (Objects.nonNull(this.transactionManager)) {
            engineConfiguration.setTransactionManager(transactionManager);
        }
        if (Objects.nonNull(this.deploymentCache)) {
            engineConfiguration.setProcessDefinitionCache(deploymentCache);
        }
    }
}
