package com.flow.controller;

import com.flow.common.core.model.PageResult;
import com.flow.common.core.model.Result;
import com.flow.entity.ReportDatasource;
import com.flow.model.ExecSql;
import com.flow.model.ReportDatasourceQuery;
import com.flow.service.ReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Set;

@RestController
@RequestMapping("/reportDatasource")
public class ReportDatasourceController {
    @Autowired
    private ReportDatasourceService reportDatasourceService;

    @GetMapping
    public Result<PageResult<ReportDatasource>> getList(ReportDatasourceQuery reportDatasourceQuery) {
        PageResult<ReportDatasource> list = reportDatasourceService.getList(reportDatasourceQuery);
        return Result.success(list);
    }

    @PreAuthorize("hasAnyAuthority('reportDatasource:add')")
    @PostMapping
    public Result<ReportDatasource> create(@RequestBody ReportDatasource reportDatasource) {
        return Result.success(reportDatasourceService.create(reportDatasource));
    }

    @PreAuthorize("hasAnyAuthority('reportDatasource:edit')")
    @PutMapping
    public Result<ReportDatasource> update(@RequestBody ReportDatasource reportDatasource) {
        return Result.success(reportDatasourceService.update(reportDatasource));
    }

    @PreAuthorize("hasAnyAuthority('reportDatasource:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        reportDatasourceService.delete(ids);
        return Result.success();
    }

    @PreAuthorize("hasAnyAuthority('reportDatasource:execSql')")
    @PostMapping("/execSql")
    public Result<?> execSql(@RequestBody ExecSql execSql) {
        return Result.success(reportDatasourceService.execSql(execSql));
    }

}
