package com.flow.delegate;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.constant.NodeTypeConstant;
import com.flow.entity.FlowActivity;
import com.flow.entity.FlowCc;
import com.flow.enums.ProcessStatus;
import com.flow.service.FlowActivityService;
import com.flow.service.FlowCcService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.JavaDelegate;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 抄送任务处理器--java服务任务
 */
@Slf4j
@Component
public class CcDelegate implements JavaDelegate {
    @Autowired
    private FlowActivityService flowActivityService;
    @Autowired
    private FlowCcService flowCcService;

    @Override
    public void execute(DelegateExecution execution) {
        LocalDateTime startTime = LocalDateTime.now();
        ExecutionEntityImpl entity = (ExecutionEntityImpl) execution;
        String activityId = execution.getCurrentActivityId();
        // 抄送
        ArrayList<String> assignees = execution.getVariable(String.format("%sCollection", activityId), ArrayList.class);
        if (CollectionUtils.isNotEmpty(assignees)) {
            log.info("抄送人:{}", assignees);
            List<FlowCc> flowCcs = assignees.stream().map(assignee -> {
                FlowCc flowCc = new FlowCc();
                flowCc.setName(entity.getActivityName());
                flowCc.setInstanceId(entity.getProcessInstanceId());
                flowCc.setDefinitionId(entity.getProcessDefinitionId());
                flowCc.setNodeId(activityId);
                flowCc.setUserId(assignee);
                flowCc.setCreateTime(LocalDateTime.now());
                return flowCc;
            }).collect(Collectors.toList());
            flowCcService.saveBatch(flowCcs);
        }
        // 记录日志
        FlowElement flowElement = entity.getCurrentFlowElement();
        List<FlowActivity> activityList = assignees.stream().map(assignee -> {
            FlowActivity flowActivity = flowActivityService.build(entity);
            flowActivity.setParentId(flowActivity.getExecutionId());
            flowActivity.setNodeType(NodeTypeConstant.CC);
            flowActivity.setNodeName(flowElement.getName());
            flowActivity.setStartTime(startTime);
            flowActivity.setEndTime(LocalDateTime.now());
            flowActivity.setUserId(assignee);
            flowActivity.setStatus(ProcessStatus.COMPLETED);
            return flowActivity;
        }).collect(Collectors.toList());
        flowActivityService.saveBatch(activityList);
    }
}
