package com.flow.delegate;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.cmd.JumpActivityCmd;
import com.flow.constant.NodeTypeConstant;
import com.flow.entity.FlowActivity;
import com.flow.entity.FlowDefine;
import com.flow.entity.node.JumpNode;
import com.flow.entity.node.Node;
import com.flow.enums.ProcessStatus;
import com.flow.model.Comment;
import com.flow.service.FlowActivityService;
import com.flow.service.FlowDefineService;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.engine.ManagementService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.JavaDelegate;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

@Slf4j
@Component
public class JumpDelegate implements JavaDelegate {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private FlowActivityService flowActivityService;
    @Autowired
    private ManagementService managementService;

    @Override
    public void execute(DelegateExecution execution) {
        ExecutionEntityImpl entity = (ExecutionEntityImpl) execution;
        FlowDefine define = flowDefineService.getDefine(entity.getProcessDefinitionId());
        JumpNode jumpNode = define.getNode(execution.getCurrentActivityId(), JumpNode.class);
        String targetNode = jumpNode.getTargetNode();
        // 记录日志
        Node target = define.getNode(jumpNode.getTargetNode(), Node.class);
        FlowElement flowElement = entity.getCurrentFlowElement();
        FlowActivity flowActivity = flowActivityService.build(entity);
        flowActivity.setParentId(flowActivity.getExecutionId());
        flowActivity.setNodeType(NodeTypeConstant.JUMP);
        flowActivity.setNodeName(flowElement.getName());
        flowActivity.setStartTime(LocalDateTime.now());
        flowActivity.setEndTime(LocalDateTime.now());
        flowActivity.setUserId("admin");
        flowActivity.setStatus(ProcessStatus.COMPLETED);
        Comment comment = new Comment();
        comment.setUserId("admin");
        comment.setTime(LocalDateTime.now());
        comment.setContent(String.format("<span>跳转至 <span class='color-primary'>%s</span></span>", target.getName()));
        comment.setAttachments(Lists.newArrayList());
        flowActivity.setComment(Lists.newArrayList(comment));
        flowActivityService.save(flowActivity);
        log.info("跳转节点:{}", targetNode);
        List<String> executionIds = managementService.executeCommand(new JumpActivityCmd(
                execution.getProcessInstanceId(),
                targetNode
        ));
        // 删除当前节点的活动记录
        if (CollectionUtils.isNotEmpty(executionIds)) {
            flowActivityService.lambdaUpdate()
                    .eq(FlowActivity::getInstanceId, execution.getProcessInstanceId())
                    .eq(FlowActivity::getStatus,  ProcessStatus.RUNNING)
                    .in(FlowActivity::getParentId, executionIds)
                    .remove();
        }
    }
}
