package com.flow.service.impl;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.dao.FlowActivityDao;
import com.flow.entity.FlowActivity;
import com.flow.entity.FlowDefine;
import com.flow.entity.node.ApprovalNode;
import com.flow.enums.ApprovalMultiEnum;
import com.flow.service.FlowActivityService;
import com.flow.service.FlowDefineService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityImpl;
import org.flowable.engine.runtime.Execution;
import org.flowable.task.service.impl.persistence.entity.TaskEntityImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.List;
import java.util.Objects;

@Service
public class FlowActivityServiceImpl extends BaseServiceImpl<FlowActivityDao, FlowActivity> implements FlowActivityService {
    @Autowired
    private FlowActivityDao flowActivityDao;
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private RuntimeService runtimeService;

    @Override
    public List<FlowActivity> record(String instanceId) {
        return this.lambdaQuery()
                .eq(FlowActivity::getInstanceId, instanceId)
                .orderByAsc(FlowActivity::getId)
                .list();
    }

    @Override
    public FlowActivity build(ExecutionEntityImpl entity) {
        FlowActivity flowActivity = new FlowActivity();
        flowActivity.setNodeId(entity.getActivityId());
        flowActivity.setNodeName(entity.getActivityName());
        if (Objects.nonNull(entity.getStartTime())) {
            LocalDateTime startTime = LocalDateTime.ofInstant(
                    entity.getStartTime().toInstant(),
                    ZoneId.systemDefault()
            );
            flowActivity.setStartTime(startTime);
        }
        flowActivity.setInstanceId(entity.getProcessInstanceId());
        flowActivity.setDefineId(entity.getProcessDefinitionId());
        flowActivity.setExecutionId(entity.getId());
        //flowActivity.setUserId(entity.getStartUserId());
        return flowActivity;
    }

    @Override
    public FlowActivity build(TaskEntityImpl entity) {
        // 获取流程定义
        FlowDefine flowDefine = flowDefineService.getDefine(entity.getProcessDefinitionId());
        ApprovalNode approvalNode = flowDefine.getNode(
                entity.getTaskDefinitionKey(),
                ApprovalNode.class
        );
        ApprovalMultiEnum multi = approvalNode.getMulti();
        // 日志
        FlowActivity flowActivity = new FlowActivity();
        String executionId = entity.getExecutionId();
        flowActivity.setExecutionId(executionId);
        if (StringUtils.isNotBlank(executionId)) {
            Execution execution = runtimeService.createExecutionQuery()
                    .executionId(executionId)
                    .singleResult();
            if (Objects.nonNull(execution)) {
                flowActivity.setParentId(execution.getParentId());
            }
        }
        flowActivity.setNodeType("approval");
        flowActivity.setMultiType(multi.getMulti());
        flowActivity.setNodeId(entity.getTaskDefinitionKey());
        flowActivity.setNodeName(entity.getName());
        if (Objects.nonNull(entity.getCreateTime())) {
            LocalDateTime startTime = LocalDateTime.ofInstant(
                    entity.getCreateTime().toInstant(),
                    ZoneId.systemDefault()
            );
            flowActivity.setStartTime(startTime);
        }
        flowActivity.setInstanceId(entity.getProcessInstanceId());
        flowActivity.setDefineId(entity.getProcessDefinitionId());
        flowActivity.setTaskId(entity.getId());
        flowActivity.setUserId(entity.getAssignee());
        return flowActivity;
    }

}
