package com.flow.controller;

import com.flow.common.core.model.PageResult;
import com.flow.common.core.model.Result;
import com.flow.entity.FlowTask;
import com.flow.entity.node.Node;
import com.flow.model.*;
import com.flow.service.FlowTaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/task")
public class FlowTaskController {
    @Autowired
    private FlowTaskService flowTaskService;

    @GetMapping("/form/{taskId}")
    public Result<FormInfo> getForm(@PathVariable String taskId) {
        FormInfo formInfo = flowTaskService.getForm(taskId);
        return Result.success(formInfo);
    }

    @GetMapping("/todo")
    public Result<PageResult<FlowTask>> todo(FlowTaskQuery flowTaskQuery) {
        return Result.success(flowTaskService.todo(flowTaskQuery));
    }

    @GetMapping("/done")
    public Result<PageResult<FlowTask>> done(FlowTaskQuery flowTaskQuery) {
        return Result.success(flowTaskService.done(flowTaskQuery));
    }

    @PostMapping("/reject")
    public Result<?> reject(@RequestBody RejectTaskForm taskForm) {
        flowTaskService.reject(taskForm);
        return Result.success();
    }

    @PostMapping("/refuse")
    public Result<?> refuse(@RequestBody TaskForm taskForm) {
        flowTaskService.refuse(taskForm);
        return Result.success();
    }

    @PostMapping("/transfer")
    public Result<?> transfer(@RequestBody TransferTaskForm taskForm) {
        flowTaskService.transfer(taskForm);
        return Result.success();
    }

    @PostMapping("/addSign")
    public Result<?> addSign(@RequestBody AddSignTaskForm taskForm) {
        flowTaskService.addSign(taskForm);
        return Result.success();
    }

    @PostMapping("/removeSign")
    public Result<?> removeSign(@RequestBody AddSignTaskForm taskForm) {
        flowTaskService.removeSign(taskForm);
        return Result.success();
    }

    @PostMapping("/delegate")
    public Result<?> delegate(@RequestBody TransferTaskForm taskForm) {
        flowTaskService.delegate(taskForm);
        return Result.success();
    }

    @PostMapping("/complete")
    public Result<?> complete(@RequestBody TaskForm taskForm) {
        flowTaskService.complete(taskForm);
        return Result.success();
    }

    @PostMapping("/taskForm")
    public Result<?> saveForm(@RequestBody TaskForm taskForm) {
        flowTaskService.saveForm(taskForm);
        return Result.success();
    }

    @PostMapping("/comment")
    public Result<?> comment(@RequestBody TaskForm taskForm) {
        flowTaskService.comment(taskForm);
        return Result.success();
    }

    @GetMapping("/parentNodes/{taskId}")
    public Result<List<Node>> getParentNodes(@PathVariable String taskId) {
        return Result.success(flowTaskService.getParentNodes(taskId));
    }

    @GetMapping("/unStartedUsers/{taskId}")
    public Result<List<String>> getMultiInstanceUnStartedUsers(@PathVariable String taskId) {
        return Result.success(flowTaskService.getMultiInstanceUnStartedUsers(taskId));
    }

}
