package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.common.core.util.ApplicationContextUtil;
import com.flow.common.oauth2.utils.SecurityContextUtil;
import com.flow.entity.User;
import com.flow.enums.AssigneeTypeEnum;
import com.flow.enums.OrganizationTypeEnum;
import com.flow.service.DeptService;
import com.flow.service.RoleService;
import com.flow.service.UserService;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FlowElement;

import java.util.*;
import java.util.stream.Collectors;

@EqualsAndHashCode(callSuper = true)
@Data
public abstract class AssigneeNode extends Node {
    // 审批对象
    private AssigneeTypeEnum assigneeType;
    // 表单内人员
    private String formUser;
    // 表单内角色
    private String formRole;
    // 审批人
    private List<String> users;
    // 审批人角色
    private List<String> roles;
    // 组织
    private List<String> organizations;
    // 表单内组织
    private String formOrganization;
    // 组织办理类型
    private OrganizationTypeEnum organizationType;
    // 组织内角色
    private List<String> organizationRoles;
    // 主管
    private Integer leader;
    // 组织主管
    private Integer orgLeader;
    // 发起人自选：true-单选，false-多选
    private Boolean choice;
    // 发起人自己
    private Boolean self;

    public abstract List<FlowElement> convert();

    public List<String> getAssignees(Map<String, Object> values) {
        UserService userService = ApplicationContextUtil.getBean(UserService.class);
        RoleService roleService = ApplicationContextUtil.getBean(RoleService.class);
        DeptService deptService = ApplicationContextUtil.getBean(DeptService.class);
        String userId = SecurityContextUtil.getUserId();
        List<String> assignees = new ArrayList<>();
        switch (this.assigneeType) {
            case USER:
                assignees.addAll(this.users);
                break;
            case SELF:
                assignees.add(userId);
                break;
            case FORM_USER:
                Object formUser = values.get(this.getFormUser());
                if (Objects.nonNull(formUser)) {
                    if (formUser instanceof Collection) {
                        assignees.addAll((Collection<? extends String>) formUser);
                    } else {
                        assignees.add(formUser.toString());
                    }
                }
                break;
            case FORM_ROLE:
                Object formRole = values.get(this.getFormRole());
                if (Objects.nonNull(formRole)) {
                    ArrayList<String> roleIds = new ArrayList<>();
                    if (formRole instanceof Collection) {
                        roleIds.addAll((Collection<? extends String>) formRole);
                    } else {
                        roleIds.add(formRole.toString());
                    }
                    List<User> userList = roleService.getUsers(roleIds);
                    if (CollectionUtils.isNotEmpty(userList)) {
                        List<String> userIds = userList.stream()
                                .map(User::getUsername)
                                .collect(Collectors.toList());
                        assignees.addAll(userIds);
                    }
                }
                break;
            case CHOICE:
                Object choiceUser = values.get(String.format("%sCollection", this.getId()));
                if (Objects.nonNull(choiceUser)) {
                    if (choiceUser instanceof Collection) {
                        assignees.addAll((Collection<? extends String>) choiceUser);
                    } else {
                        assignees.add(choiceUser.toString());
                    }
                }
                break;
            case LEADER:
                User leader = userService.getLeader(userId, this.leader);
                if (Objects.nonNull(leader)) {
                    assignees.add(leader.getUsername());
                }
                break;
            case ORG:
                OrganizationTypeEnum organizationTypeEnum = this.getOrganizationType();
                List<User> leaderList = Lists.newArrayList();
                if (organizationTypeEnum == OrganizationTypeEnum.LEADER) {
                    leaderList = deptService.getLeader(this.getOrganizations());
                }
                if (organizationTypeEnum == OrganizationTypeEnum.ALL) {
                    leaderList = deptService.getUsers(this.getOrganizations());
                }
                if (organizationTypeEnum == OrganizationTypeEnum.ROLE) {
                    leaderList =deptService.getRoleUsers(this.getOrganizations(), this.getOrganizationRoles());
                }
                if (CollectionUtils.isNotEmpty(leaderList)) {
                    List<String> userIds = leaderList.stream().map(User::getUsername).collect(Collectors.toList());
                    assignees.addAll(userIds);
                }
                break;
            case FORM_ORG:
                Object formOrg = values.get(this.getFormOrganization());
                if (Objects.nonNull(formOrg)) {
                    ArrayList<String> orgIds = new ArrayList<>();
                    if (formOrg instanceof Collection) {
                        orgIds.addAll((Collection<? extends String>) formOrg);
                    } else {
                        orgIds.add(formOrg.toString());
                    }
                    OrganizationTypeEnum organizationType = this.getOrganizationType();
                    List<User> userList = Lists.newArrayList();
                    if (organizationType == OrganizationTypeEnum.LEADER) {
                        userList = deptService.getLeader(this.getOrganizations());
                    }
                    if (organizationType == OrganizationTypeEnum.ALL) {
                        userList = deptService.getUsers(this.getOrganizations());
                    }
                    if (organizationType == OrganizationTypeEnum.ROLE) {
                        userList = deptService.getRoleUsers(this.getOrganizations(), this.getOrganizationRoles());
                    }
                    if (CollectionUtils.isNotEmpty(userList)) {
                        List<String> userIds = userList.stream()
                                .map(User::getUsername)
                                .collect(Collectors.toList());
                        assignees.addAll(userIds);
                    }
                }
                break;
            case ORG_LEADER:
                User orgLeader = userService.getOrgLeader(userId, this.orgLeader);
                if (Objects.nonNull(orgLeader)) {
                    assignees.add(orgLeader.getUsername());
                }
                break;
            case ROLE:
                List<User> users = roleService.getUsers(this.roles);
                if (CollectionUtils.isNotEmpty(users)) {
                    List<String> userIds = users.stream()
                            .map(User::getUsername)
                            .collect(Collectors.toList());
                    assignees.addAll(userIds);
                }
                break;
        }
        return assignees;
    }
}
